from oterm.store.upgrades.v0_1_6 import upgrades as v0_1_6_upgrades
from oterm.store.upgrades.v0_1_11 import upgrades as v_0_1_11_upgrades
from oterm.store.upgrades.v0_2_0 import upgrades as v0_2_0_upgrades
from oterm.store.upgrades.v0_2_4 import upgrades as v0_2_4_upgrades
from oterm.store.upgrades.v0_2_8 import upgrades as v0_2_8_upgrades
from oterm.store.upgrades.v0_3_0 import upgrades as v0_3_0_upgrades
from oterm.store.upgrades.v0_4_0 import upgrades as v0_4_0_upgrades
from oterm.store.upgrades.v0_5_1 import upgrades as v0_5_1_upgrades
from oterm.store.upgrades.v0_6_0 import upgrades as v0_6_0_upgrades
from oterm.store.upgrades.v0_7_0 import upgrades as v0_7_0_upgrades
from oterm.store.upgrades.v0_9_0 import upgrades as v0_9_0_upgrades
from oterm.store.upgrades.v0_12_0 import upgrades as v0_12_0_upgrades
from oterm.store.upgrades.v0_13_1 import upgrades as v0_13_1_upgrades

upgrades = (
    v0_1_6_upgrades
    + v_0_1_11_upgrades
    + v0_2_0_upgrades
    + v0_2_4_upgrades
    + v0_2_8_upgrades
    + v0_3_0_upgrades
    + v0_4_0_upgrades
    + v0_5_1_upgrades
    + v0_6_0_upgrades
    + v0_7_0_upgrades
    + v0_9_0_upgrades
    + v0_12_0_upgrades
    + v0_13_1_upgrades
)
